from sqlalchemy             import engine_from_config
from sqlalchemy.orm         import scoped_session
from sqlalchemy.orm         import sessionmaker
from zope.sqlalchemy        import ZopeTransactionExtension
from paste.deploy.loadwsgi  import appconfig
from pkg_resources          import resource_filename
from pyramid                import testing
from pyramid.authentication import AuthTktAuthenticationPolicy
from pyramid.authorization  import ACLAuthorizationPolicy
from pyramid.response       import Response
from hem.interfaces         import IDBSession
from horus.interfaces       import IHorusUserClass
from horus.interfaces       import IHorusActivationClass
from hiero.interfaces       import IHieroEntryClass
from hiero.tests.models     import Base
from hiero.tests.models     import User
from hiero.tests.models     import Activation
from hiero.tests.models     import Entry
from pyramid_beaker         import session_factory_from_settings
from webtest                import TestApp

import unittest

settings = appconfig('config:' + resource_filename(__name__, 'test.ini'))


class BaseTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.engine = engine_from_config(settings, prefix='sqlalchemy.')
        cls.Session = sessionmaker()

    def setUp(self):
        self.config = testing.setUp()

        self.connection = connection = self.engine.connect()

        # begin a non-ORM transaction
        self.trans = connection.begin()

        # bind an individual Session to the connection
        self.session = self.Session(bind=connection)

        self.config.registry.registerUtility(self.session, IDBSession)
        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Entry, IHieroEntryClass)

        Base.metadata.bind=connection

    def tearDown(self):
        # rollback - everything that happened with the
        # Session above (including calls to commit())
        # is rolled back.
        testing.tearDown()
        self.trans.rollback()
        self.session.close()

DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))

class IntegrationTestBase(unittest.TestCase):
    def main(self, global_config, **settings):
        config = global_config
        config.add_settings(settings)

        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)

        authz_policy = ACLAuthorizationPolicy()
        config.set_authorization_policy(authz_policy)


        authn_policy = AuthTktAuthenticationPolicy('secret')
        config.set_authentication_policy(authn_policy)

        session_factory = session_factory_from_settings(settings)

        config.set_session_factory(session_factory)

        config.registry.registerUtility(DBSession, IDBSession)

        config.include('hiero')

        app = config.make_wsgi_app()

        return app

    def setUp(self):
        self.engine = engine_from_config(settings, prefix='sqlalchemy.')
        self.config = testing.setUp()
        app = self.main(self.config, **settings)
        self.app = TestApp(app)
        self.connection = connection = self.engine.connect()

        self.session = app.registry.getUtility(IDBSession)
        self.session.configure(bind=connection)
        # begin a non-ORM transaction
        self.trans = connection.begin()

        Base.metadata.bind=connection

    def tearDown(self):
        # rollback - everything that happened with the
        # Session above (including calls to commit())
        # is rolled back.
        testing.tearDown()
        self.trans.rollback()
        self.session.close()



# -*- coding: utf-8 -*-
"""
    The Pygments reStructuredText directive
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This fragment is a Docutils_ 0.5 directive that renders source code
    (to HTML only, currently) via Pygments.

    To use it, adjust the options below and copy the code into a module
    that you import on initialization.  The code then automatically
    registers a ``sourcecode`` directive that you can use instead of
    normal code blocks like this::

        .. sourcecode:: python

            My code goes here.

    If you want to have different code styles, e.g. one with line numbers
    and one without, add formatters with their names in the VARIANTS dict
    below.  You can invoke them instead of the DEFAULT one by using a
    directive option::

        .. sourcecode:: python
            :linenos:

            My code goes here.

    Look at the `directive documentation`_ to get all the gory details.

    .. _Docutils: http://docutils.sf.net/
    .. _directive documentation:
       http://docutils.sourceforge.net/docs/howto/rst-directives.html

    :copyright: Copyright 2006-2010 by the Pygments team, see AUTHORS.
    :license: BSD, see LICENSE for details.
"""

# Options
# ~~~~~~~

# Set to True if you want inline CSS styles instead of classes
INLINESTYLES = False

from pygments.formatters import HtmlFormatter
import docutils.core

# The default formatter
DEFAULT = HtmlFormatter(noclasses=INLINESTYLES)

# Add name -> formatter pairs for every variant you want to use
VARIANTS = {
    # 'linenos': HtmlFormatter(noclasses=INLINESTYLES, linenos=True),
}


from docutils import nodes
from docutils.parsers.rst import directives, Directive

from pygments import highlight
from pygments.lexers import get_lexer_by_name, TextLexer

class Pygments(Directive):
    """ Source code syntax hightlighting.
    """
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = dict([(key, directives.flag) for key in VARIANTS])
    has_content = True

    def run(self):
        self.assert_has_content()
        try:
            lexer = get_lexer_by_name(self.arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = TextLexer()
        # take an arbitrary option if more than one is given
        formatter = self.options and VARIANTS[self.options.keys()[0]] or DEFAULT
        parsed = highlight(u'\n'.join(self.content), lexer, formatter)
        return [nodes.raw('', parsed, format='html')]

directives.register_directive('sourcecode', Pygments)

class RestructuredTextFormatter(object):
    def __init__(self, text):
        self.text = text

    def get_html(self):
        parts = docutils.core.publish_parts(self.text, writer_name="html") 
        html = parts['body']
        return html



from hiero.interfaces   import IHieroEntryClass
from hiero.interfaces   import IHieroSeriesClass
from hiero.interfaces   import IHieroCategoryClass

from hem.config         import get_class_from_config

def includeme(config):
    settings = config.registry.settings

    if not config.registry.queryUtility(IHieroEntryClass):
        entry_class = get_class_from_config(settings, 'hiero.entry_class')
        config.registry.registerUtility(entry_class, IHieroEntryClass)

    if not config.registry.queryUtility(IHieroSeriesClass):
        series_class = get_class_from_config(settings, 'hiero.series_class')
        config.registry.registerUtility(series_class, IHieroSeriesClass)

    if not config.registry.queryUtility(IHieroCategoryClass):
        cat_class = get_class_from_config(settings, 'hiero.category_class')
        config.registry.registerUtility(cat_class, IHieroCategoryClass)

    config.include('hiero.routes')

 #   config.add_route('get_pages',   '/pages') 
 #   config.add_route('get_page',    '/pages/{link_title}')
 #   config.add_route('edit_page',   '/pages/{link_title}/edit')
 #   config.add_route('add_page',    '/add_page')
 #   config.add_route('remove_page', '/pages/{link_title}/remove')
    config.add_subscriber(add_renderer_globals, 'pyramid.events.BeforeRender')
    config.scan()
    return config.make_wsgi_app()


def add_renderer_globals(event):
    event['stylesheet_link_tag'] = stylesheet_link_tag
    event['script_tag'] = script_tag
    request = event['request']
    cat_class = request.registry.getUtility(IHieroCategoryClass)
    ser_class = request.registry.getUtility(IHieroSeriesClass)
    ent_class = request.registry.getUtility(IHieroEntryClass)
    event['categories'] = cat_class.get_all(request)
    event['series'] = ser_class.get_all(request)
    event['recent_entries'] = ent_class.get_all_active(request, limit=5)

def stylesheet_link_tag(request, url):
  full_url = request.static_url("hiero:static%s" % url)
  return "<link rel=\"stylesheet\" href=\"%s\" type=\"text/css\" media=\"screen\" charset=\"utf-8\" />" % full_url

def script_tag(request, url):
  full_url = request.static_url("hiero:static%s" % url)
  return "<script type=\"text/javascript\" src=\"%s\" charset=\"utf-8\" ></script>" % full_url



import deform

class BaseForm(deform.Form):
    def __init__(self, request, *args, **kwargs):
        self.request = request

        if not kwargs.get('buttons'):
            kwargs['buttons'] = ('submit', )

        super(BaseForm, self).__init__(*args, **kwargs)

class BootstrapForm(BaseForm):
    """ This form renders out twitter bootstrap templates """
    def __init__(self, *args, **kwargs):
        super(BootstrapForm, self).__init__(*args, **kwargs)

        for child in self.children:
            if isinstance(child.widget, deform.widget.TextInputWidget) or \
                isinstance(child.widget, deform.widget.TextAreaWidget):

                if not child.widget.css_class:
                    child.widget.css_class = ''

                if not 'xlarge' in child.widget.css_class:
                    child.widget.css_class += ' xlarge'

class HieroForm(BootstrapForm):
    """ This is the standard form we should use through out our code, that way
    if we decide to swap our rendering later, we only have to do it in 1 place
    """
    pass




from hiero.resources    import EntryFactory
from hiero.resources    import CategoryFactory
from hiero.resources    import SeriesFactory

def includeme(config):
    # non-admin routes
    config.add_route('hiero_entry_index',       '')
    config.add_route('hiero_entry_index_paged', '/page/{page}')
    config.add_route('hiero_entry_detail',      '/detail/{slug}')
    config.add_route('hiero_entry_search',      '/search/{term}')
    config.add_route('hiero_entry_category',    '/category/{slug}')
    config.add_route('hiero_entry_series',      '/series/{slug}')

    # admin routes
    config.add_route('hiero_admin_index',   '/admin')
    config.add_route('hiero_admin_entry_index_paged',   '/admin/entries/page/{page}')
    config.add_route('hiero_admin_entry_index',   '/admin/entries')
    config.add_route('hiero_admin_entry_create',   '/admin/entries/new')
    config.add_route('hiero_admin_entry_edit'
           , '/admin/entries/{slug}/edit'
           , factory=EntryFactory
           , traverse="/{slug}"
    )

    config.add_route('hiero_admin_category_index_paged',
            '/admin/categories/page/{page}')
    config.add_route('hiero_admin_category_index',   '/admin/categories')
    config.add_route('hiero_admin_category_create',   '/admin/categories/new')
    config.add_route('hiero_admin_category_edit'
           , '/admin/categories/{slug}/edit'
           , factory=CategoryFactory
           , traverse="/{slug}"
    )

    config.add_route('hiero_admin_series_index_paged',
            '/admin/series/page/{page}')
    config.add_route('hiero_admin_series_index',   '/admin/series')
    config.add_route('hiero_admin_series_create',   '/admin/series/new')
    config.add_route('hiero_admin_series_edit'
           , '/admin/series/{slug}/edit'
           , factory=SeriesFactory
           , traverse="/{slug}"
    )


    config.add_static_view('static/deform', 'deform:static')
    config.add_static_view('static', 'hiero:static')



import os
import sys
import glob

import inspect

def load_modules(filemask='*.py', ignore_list=('__init__.py', )):
    """
    This loads all modules from the formatters directory
    """
    modules = {}
    dirname = os.path.dirname(__file__)

    if dirname:
        filemask = os.path.join(dirname, filemask)

    for fn in glob.glob(filemask):
        fn = os.path.split(fn)[1]

        if fn in ignore_list:
            continue

        fn = os.path.splitext(fn)[0]
        modules[fn] = __import__(fn, globals(), locals())

    return modules


def get_formatter(formatter_class):
    """
    This will generate HTML for a specific markup language but will not
    highlight source code
    """
    cls = ''

    if formatter_class == 'rst':
        cls = 'RestructuredTextFormatter'
    elif formatter_class == 'markdown':
        cls = 'MarkdownFormatter'

    load_modules()
    current_module = sys.modules[__name__]
    formatter = get_module_classes(current_module, cls)

    return formatter


def get_module_classes(module, formatter_class):
    """
    Recursively searches through all modules looking for the class
    that we are asking for
    """
    for name, obj in inspect.getmembers(module, inspect.isclass):
        if obj.__name__ == formatter_class:
            return obj

    # Didn't find the class we want on that module, loop thorugh
    # its child modules
    for name, obj in inspect.getmembers(module, inspect.ismodule):
        if obj.__name__.startswith(__name__):
            cls = get_module_classes(obj, formatter_class)

            if cls:
                return cls



class LinebreaksFormatter(object):
    def __init__(self, text):
        self.text = text

    def get_html(self):
        """
        This parses out new lines and replaces them with html breaks
        """
        return self.text.replace('\n', '<br />')



import markdown


class MarkdownFormatter(object):
    def __init__(self, text):
        self.text = text

    def get_html(self):
        return markdown.markdown(self.text, ['codehilite(force_linenos=False)'])



from hiero.tests        import BaseTestCase
from sqlalchemy.types   import DateTime
from sqlalchemy         import Column
from datetime           import datetime
from hiero.tests.models import Base

class TestModel(Base):
    start_date = Column(DateTime)

class TestModels(BaseTestCase):
    def test_tablename(self):
        model = TestModel()
        assert model.__tablename__ == 'test_model'

    def test_json(self):
        model = TestModel()
        model.pk = 1
        model.start_date = datetime.now()

        assert model.__json__() == {'pk': 1, 'start_date': model.start_date.isoformat()}

class TestEntry(BaseTestCase):
    def test_create_entry(self):
        from hiero.tests.models import User
        from hiero.tests.models import Entry
        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')
        self.session.add(owner)
        self.session.flush()

        entry = Entry(owner_pk=owner.pk, title='test entry', content='hi',
            html_content='hi<br />'
        )

        self.session.add(entry)
        self.session.flush()

        assert entry.slug == 'test-entry'
        assert entry.is_published == False
        assert entry.is_featured == False
        assert entry.enable_comments == False
        assert entry.owner != None
        assert owner.entries[0] == entry

    def test_entry_tags(self):
        from hiero.tests.models import User
        from hiero.tests.models import Entry
        from hiero.tests.models import Tag

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />'
        )

        tag1 = Tag(title='python')
        tag2 = Tag(title='linux')

        entry.tags.append(tag1)
        entry.tags.append(tag2)

        self.session.add(entry)
        self.session.flush()

        assert len(entry.tags) == 2
        assert tag1.entries[0] == entry
        assert tag2.entries[0] == entry

    def test_entry_related_entries(self):
        from hiero.tests.models import User
        from hiero.tests.models import Entry

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />'
        )

        entry2 = Entry(owner=owner, title='test entry 1', content='hi',
            html_content='hi<br />'
        )

        entry3 = Entry(owner=owner, title='test entry 2', content='hi',
            html_content='hi<br />'
        )

        entry.related_entries.append(entry2)
        entry.related_entries.append(entry3)

        self.session.add(entry)

        self.session.flush()

        assert len(entry.related_entries) == 2
        assert entry.related_entries[0] == entry2
        assert entry.related_entries[1] == entry3

    def test_entry_entry_series(self):
        from hiero.tests.models import User
        from hiero.tests.models import Entry
        from hiero.tests.models import Series

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />'
        )

        entry2 = Entry(owner=owner, title='test entry 1', content='hi',
            html_content='hi<br />'
        )

        entry3 = Entry(owner=owner, title='test entry 2', content='hi',
            html_content='hi<br />'
        )

        series = Series(title='test series')
        series.entries.append(entry)
        series.entries.append(entry2)
        series.entries.append(entry3)


        self.session.add(series)

        self.session.flush()

        assert entry.series == series
        assert len(series.entries) == 3 
        assert series.entries[0] == entry
        assert series.entries[1] == entry2
        assert series.entries[2] == entry3
        assert series.slug == 'test-series'

    def test_entry_category(self):
        from hiero.tests.models import User
        from hiero.tests.models import Entry
        from hiero.tests.models import Category

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />'
        )

        entry2 = Entry(owner=owner, title='test entry 1', content='hi',
            html_content='hi<br />'
        )

        entry3 = Entry(owner=owner, title='test entry 2', content='hi',
            html_content='hi<br />'
        )

        category = Category(title='test category')
        category.entries.append(entry)
        category.entries.append(entry2)
        category.entries.append(entry3)


        self.session.add(category)

        self.session.flush()

        assert entry.category == category
        assert len(category.entries) == 3
        assert category.entries[0] == entry
        assert category.entries[1] == entry2
        assert category.entries[2] == entry3
        assert category.slug == 'test-category'







from horus.resources import RootFactory
from hiero.interfaces   import IHieroEntryClass
from hiero.interfaces   import IHieroCategoryClass
from hiero.interfaces   import IHieroSeriesClass

class EntryFactory(RootFactory):
    def __init__(self, request):
        self.request = request
        self.Entry = request.registry.getUtility(IHieroEntryClass)

    def __getitem__(self, key):
        entry = self.Entry.get_by_slug(self.request, key).one()

        if entry:
            entry.__parent__ = self
            entry.__name__ = key

        return entry

class CategoryFactory(RootFactory):
    def __init__(self, request):
        self.request = request
        self.Category = request.registry.getUtility(IHieroCategoryClass)

    def __getitem__(self, key):
        category = self.Category.get_by_slug(self.request, key).one()

        if category:
            category.__parent__ = self
            category.__name__ = key

        return category

class SeriesFactory(RootFactory):
    def __init__(self, request):
        self.request = request
        self.Series = request.registry.getUtility(IHieroSeriesClass)

    def __getitem__(self, key):
        series = self.Series.get_by_slug(self.request, key).one()

        if series:
            series.__parent__ = self
            series.__name__ = key

        return series



from hiero.tests    import BaseTestCase
from hiero.tests    import IntegrationTestBase
from pyramid        import testing
from mock           import Mock

class TestBlogController(BaseTestCase):
    def test_index_inactive(self):
        from hem.db                 import IDBSession
        from hiero.views.blog       import EntryController
        from horus.interfaces       import IHorusUserClass
        from horus.interfaces       import IHorusActivationClass
        from hiero.interfaces       import IHieroEntryClass
        from hiero.tests.models     import User
        from hiero.tests.models     import Entry
        from hiero.tests.models     import Activation

        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Entry, IHieroEntryClass)
        self.config.registry.registerUtility(self.session, IDBSession)

        self.config.add_route('hiero_entry_index', '/')

        request = testing.DummyRequest()

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        self.session.add(owner)

        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />'
        )

        entry1 = Entry(owner=owner, title='test entry 2', content='hi',
            html_content='hi<br />'
        )

        self.session.add(entry)
        self.session.add(entry1)

        self.session.flush()

        controller = EntryController(request)

        results = controller.index()

        entries = results['entries']

        assert len(entries) == 0

    def test_index_active(self):
        from hem.db                 import IDBSession
        from hiero.views.blog       import EntryController
        from horus.interfaces       import IHorusUserClass
        from horus.interfaces       import IHorusActivationClass
        from hiero.interfaces       import IHieroEntryClass
        from hiero.tests.models     import User
        from hiero.tests.models     import Entry
        from hiero.tests.models     import Activation

        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Entry, IHieroEntryClass)
        self.config.registry.registerUtility(self.session, IDBSession)

        self.config.add_route('hiero_entry_index', '/')

        request = testing.DummyRequest()

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        self.session.add(owner)

        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />', is_published=True
        )

        entry1 = Entry(owner=owner, title='test entry 2', content='hi',
            html_content='hi<br />', is_published=True
        )

        self.session.add(entry)
        self.session.add(entry1)

        self.session.flush()

        controller = EntryController(request)

        results = controller.index()

        entries = results['entries']

        assert len(entries) == 2
        assert entries[0] == entry
        assert entries[1] == entry1

    def test_index_active_paging(self):
        from hem.db                 import IDBSession
        from hiero.views.blog       import EntryController
        from horus.interfaces       import IHorusUserClass
        from horus.interfaces       import IHorusActivationClass
        from hiero.interfaces       import IHieroEntryClass
        from hiero.tests.models     import User
        from hiero.tests.models     import Entry
        from hiero.tests.models     import Activation

        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Entry, IHieroEntryClass)
        self.config.registry.registerUtility(self.session, IDBSession)

        self.config.add_route('hiero_entry_index', '/')

        request = testing.DummyRequest()
        request.matchdict = Mock()

        def get(key, default):
            if key == 'page':
                return 2

        request.matchdict.get = get

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        self.session.add(owner)

        entries = []

        for i in xrange(1, 21):
            entry = Entry(owner=owner, title='test entry %s' % i, content='hi',
                html_content='hi<br />%s' % i, is_published=True,
                published_on='4/%s/2012' % i
            )

            self.session.add(entry)

            entries.append(entry)

        self.session.flush()

        controller = EntryController(request)

        results = controller.index()

        result_entries = results['entries']

        assert len(result_entries) == 10
        assert entries[12].title == result_entries[2].title
        assert not entries[0] in result_entries

    def test_detail(self):
        from hem.db                 import IDBSession
        from hiero.views.blog       import EntryController
        from horus.interfaces       import IHorusUserClass
        from horus.interfaces       import IHorusActivationClass
        from hiero.interfaces       import IHieroEntryClass
        from hiero.tests.models     import User
        from hiero.tests.models     import Entry
        from hiero.tests.models     import Activation

        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(Entry, IHieroEntryClass)
        self.config.registry.registerUtility(self.session, IDBSession)

        self.config.add_route('hiero_entry_index', '/')

        request = testing.DummyRequest()
        request.matchdict = Mock()

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')

        self.session.add(owner)

        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />', is_published=True
        )

        entry1 = Entry(owner=owner, title='test entry 2', content='hi',
            html_content='hi<br />', is_published=True
        )

        self.session.add(entry)
        self.session.add(entry1)

        self.session.flush()

        def get(key, default):
            if key == 'slug':
                return entry1.slug

        request.matchdict.get = get

        controller = EntryController(request)

        results = controller.detail()

        returned_entry = results['entry']
        returned_entry == entry

class TestBlogIntegrationViews(IntegrationTestBase):
    def test_index(self):
        """ Call the index view, make sure routes are working """
        from hiero.tests.models     import User
        from hiero.tests.models     import Entry

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')


        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />foo', is_published=True
        )

        entry1 = Entry(owner=owner, title='test entry 1', content='hi',
            html_content='hi<br />bar', is_published=True
        )

        self.session.add(owner)
        self.session.add(entry)
        self.session.add(entry1)

        res = self.app.get('/')
        assert res.status_int == 200
        assert 'foo' in res.body
        assert 'bar' in res.body

    def test_detail(self):
        """ Call the detail view, make sure routes are working """
        from hiero.tests.models     import User
        from hiero.tests.models     import Entry

        owner = User(user_name='sontek', email='sontek@gmail.com')
        owner.set_password('foo')


        entry = Entry(owner=owner, title='test entry', content='hi',
            html_content='hi<br />', is_published=True
        )

        self.session.add(owner)
        self.session.add(entry)
        self.session.flush()

        res = self.app.get('/test-entry')
        assert res.status_int == 200
        assert 'hi' in res.body



from hiero.tests            import BaseTestCase
from hiero.views            import BaseController
from pyramid                import testing

class TestViewController(BaseTestCase):
    def test_init(self):
        from hem.db                 import IDBSession
        from horus.interfaces       import IHorusUserClass
        from horus.interfaces       import IHorusActivationClass

        self.config.registry.registerUtility(Activation, IHorusActivationClass)
        self.config.registry.registerUtility(User, IHorusUserClass)
        self.config.registry.registerUtility(self.session, IDBSession)

        request = testing.DummyRequest()
        controller = BaseController(request)

        assert controller.Entry == Entry



from hiero.models               import BaseModel
from hiero.models               import UserMixin
from hiero.models               import ENTRY_ASSOCIATION_TABLE_NAME
from hiero.models               import ENTRY_TAG_TABLE_NAME
from hiero.models               import ENTRY_TABLE_NAME
from hiero.models               import SERIES_TABLE_NAME
from hiero.models               import ENTRY_TAG_ASSOCIATION_TABLE_NAME
from hem.db                     import get_session
from hem.text                   import slugify
from hem.text                   import pluralize
from sqlalchemy.ext.declarative import declared_attr

import sqlalchemy as sa

class EntryAssociationMixin(BaseModel):
    @declared_attr
    def __tablename__(cls):
        return ENTRY_ASSOCIATION_TABLE_NAME

    @declared_attr
    def parent_entry_pk(self):
        return sa.Column(sa.Integer
                , sa.ForeignKey('%s.pk' % ENTRY_TABLE_NAME
                    , onupdate='CASCADE'
                    , ondelete='CASCADE'
                )
                , primary_key=True
        )

    @declared_attr
    def related_entry_pk(self):
        return sa.Column(sa.Integer
                , sa.ForeignKey('%s.pk' % ENTRY_TABLE_NAME
                    , onupdate='CASCADE'
                    , ondelete='CASCADE'
                )
                , primary_key=True
        )

class EntryTagMixin(BaseModel):
    @declared_attr
    def __tablename__(cls):
        return ENTRY_TAG_ASSOCIATION_TABLE_NAME

    @declared_attr
    def tag_pk(self):
        return sa.Column(sa.Integer
                , sa.ForeignKey('%s.pk' % ENTRY_TAG_TABLE_NAME
                    , onupdate='CASCADE'
                    , ondelete='CASCADE'
                )
                , primary_key=True
        )

    @declared_attr
    def entry_pk(self):
        return sa.Column(sa.Integer
                , sa.ForeignKey('%s.pk' % ENTRY_TABLE_NAME
                    , onupdate='CASCADE'
                    , ondelete='CASCADE'
                )
                , primary_key=True
        )

class TagMixin(BaseModel):
    @declared_attr
    def __tablename__(cls):
        return ENTRY_TAG_TABLE_NAME

    @declared_attr
    def title(self):
        """ Unique title for the tag """
        return sa.Column(sa.Unicode(128), nullable=False, unique=True)

class CategoryMixin(BaseModel):
    @declared_attr
    def title(self):
        """ Unique title for the category """
        return sa.Column(sa.Unicode(128), nullable=False, unique=True)

    @declared_attr
    def slug(self):
        """ Unique url for the category """
        def slug_title(context):
            title = context.current_parameters['title']

            if title:
                return slugify(title)
            else:
                #TODO: need to raise IntegrityError here
                pass

        return sa.Column(
            sa.Unicode(128)
            , nullable=False
            , unique=True
            , default=slug_title
        )

    @classmethod
    def get_by_slug(cls, request, slug):
        """Gets a category by its slug """
        session = get_session(request)

        return session.query(cls).filter(cls.slug == slug)


class EntryMixin(BaseModel):
    @declared_attr
    def __tablename__(cls):
        return ENTRY_TABLE_NAME

    @declared_attr
    def owner_pk(self):
        return sa.Column(
            sa.Integer
            , sa.ForeignKey('%s.pk' % UserMixin.__tablename__)
            , nullable=False
        )

    @declared_attr
    def owner(self):
        return sa.orm.relationship(
            'User'
            , backref=pluralize(ENTRY_TABLE_NAME)
        )

    @declared_attr
    def markup(self):
        """ The markup format"""
        return sa.Column(sa.Unicode(128))

    @declared_attr
    def series_pk(self):
        return sa.Column(
            sa.Integer,
            sa.ForeignKey('%s.pk' % SERIES_TABLE_NAME)
        )

    @declared_attr
    def series(self):
        return sa.orm.relationship(
            'Series'
            , backref=pluralize(ENTRY_TABLE_NAME)
        )

    @declared_attr
    def category_pk(self):
        return sa.Column(
            sa.Integer
            , sa.ForeignKey('%s.pk' % CategoryMixin.__tablename__)
        )

    @declared_attr
    def category(self):
        return sa.orm.relationship(
            'Category'
            , backref=pluralize(ENTRY_TABLE_NAME)
        )

    @declared_attr
    def is_featured(self):
        """ Allows to pull entries to the top """
        return sa.Column(
            sa.Boolean
            , nullable=False
            , default=False
            , server_default='false'
        )

    @declared_attr
    def is_published(self):
        """ Marks if the entry is a draft or ready to be displayed """
        return sa.Column(
            sa.Boolean
            , nullable=False
            , default=False
            , server_default='false'
        )

    @declared_attr
    def enable_comments(self):
        """ If we should allow comments or not"""
        return sa.Column(
            sa.Boolean
            , nullable=False
            , default=False
            , server_default='false'
        )

    @declared_attr
    def slug(self):
        """ Unique title for the entry """
        def slug_title(context):
            title = context.current_parameters['title']

            if title:
                return slugify(title)
            else:
                #TODO: need to raise IntegrityError here
                pass

        return sa.Column(
            sa.Unicode(128)
            , nullable=False
            , unique=True
            , default=slug_title
        )

    @declared_attr
    def content(self):
        """ The non-rendered content of the entry"""
        return sa.Column(sa.UnicodeText, nullable=False)

    @declared_attr
    def html_content(self):
        """ The HTML content of the entry"""
        return sa.Column(sa.UnicodeText, nullable=False)

    @declared_attr
    def title(self):
        """ Unique title for the entry """
        return sa.Column(sa.Unicode(128), nullable=False, unique=True)

    @declared_attr
    def created_on(self):
        """ Date the entry was created """
        return sa.Column(sa.TIMESTAMP(timezone=False),
            default=sa.sql.func.now(),
            server_default=sa.func.now()
        )

    @declared_attr
    def published_on(self):
        """ Date the entry was published """
        return sa.Column(sa.DateTime(timezone=False))

    @declared_attr
    def tags(self):
        return sa.orm.relationship(
            'Tag'
            , secondary=ENTRY_TAG_ASSOCIATION_TABLE_NAME
            , passive_deletes=True
            , passive_updates=True
            , backref=pluralize(ENTRY_TABLE_NAME)
        )


    @declared_attr
    def related_entries(self):
        return sa.orm.relationship(
            'Entry'
            , secondary=ENTRY_ASSOCIATION_TABLE_NAME
            , passive_deletes=True
            , passive_updates=True
            , primaryjoin='Entry.pk == EntryAssociation.parent_entry_pk'
            , secondaryjoin='Entry.pk == EntryAssociation.related_entry_pk'
        )

    @classmethod
    def get_all(cls, request, page=None, limit=None):
        session = get_session(request)

        query = session.query(cls)
        query = query.order_by(cls.published_on.desc())

        if limit:
            query = query.limit(limit)

        if page and limit:
            offset = (page - 1) * limit
            query = query.offset(offset)

        return query

    @classmethod
    def get_all_active(cls, request, page=1, limit=10):
        """Gets all active entries"""
        session = get_session(request)

        query = cls.get_all(request, page=page, limit=limit)
        query = query.from_self().filter(cls.is_published == True)

        return query

    @classmethod
    def get_by_slug(cls, request, slug):
        """Gets an entry by its slug """
        session = get_session(request)

        return session.query(cls).filter(cls.slug == slug)

class SeriesMixin(BaseModel):
    """ This represents a series of blog entries, such a 6 entry 
    series on debugging python
    """
    @declared_attr
    def __tablename__(cls):
        return SERIES_TABLE_NAME

    @declared_attr
    def title(self):
        """ Unique title for the series """
        return sa.Column(sa.Unicode(128), nullable=False, unique=True)

    @declared_attr
    def slug(self):
        """ Unique url readable title for the series """
        def slug_title(context):
            title = context.current_parameters['title']

            if title:
                return slugify(title)
            else:
                #TODO: need to raise IntegrityError here
                pass

        return sa.Column(
            sa.Unicode(128)
            , nullable=False
            , unique=True
            , default=slug_title
        )

    @declared_attr
    def description(self):
        return sa.Column(sa.UnicodeText())

    @classmethod
    def get_by_slug(cls, request, slug):
        """Gets all active entries"""
        session = get_session(request)

        return session.query(cls).filter(cls.slug == slug)



from sqlalchemy.ext.declarative import declarative_base
from hiero.models               import GroupMixin
from hiero.models               import UserMixin
from hiero.models               import UserGroupMixin
from hiero.models               import ActivationMixin
from hiero.models               import BaseModel
from hiero.models.blog          import CategoryMixin
from hiero.models.blog          import EntryAssociationMixin
from hiero.models.blog          import EntryMixin
from hiero.models.blog          import EntryTagMixin
from hiero.models.blog          import TagMixin
from hiero.models.blog          import SeriesMixin

Base = declarative_base(cls=BaseModel)

class User(UserMixin, Base):
    pass

class Group(GroupMixin, Base):
    pass

class UserGroup(UserGroupMixin, Base):
    pass

class Activation(ActivationMixin, Base):
    pass

class EntryTag(EntryTagMixin, Base):
    pass

class Tag(TagMixin, Base):
    pass

class Category(CategoryMixin, Base):
    pass

class Entry(EntryMixin, Base):
    pass

class EntryAssociation(EntryAssociationMixin, Base):
    pass

class Series(SeriesMixin, Base):
    pass



from hiero.tests import BaseTestCase

class TestInterfaces(BaseTestCase):
    def test_entry_class(self):
        """ Shouldn't be able to instantiate the interface """
        from hiero.interfaces import IHieroEntryClass

        def make_session():
            IHieroEntryClass('1')

        self.assertRaises(TypeError, make_session)



# import re
# import code
# import transaction

# from pyramid.httpexceptions import (
#      HTTPFound
#     ,HTTPNotFound
#     )

# from pyramid.view import (
#      view_config
#     ,forbidden_view_config
#     )

# from ..models import (
#     DBSession
#     )

# from ..models.page import (
#     Page
#     )

# def escapeNewlines(s):
#   return s.replace("\n", "\\n")



# @view_config (route_name='home',
#               renderer='index.mako')
# def home(request):
#   return {
#       "logged_in": True
#       }

# @view_config (route_name='get_pages',    
#               renderer='json')
# def get_pages(request):
#   pages = DBSession.query(Page).all()
#   result = []
#   for page in pages:
#     result.append({
#       "id": page.id,
#       "link_title": page.link_title,
#       "url": request.route_url('get_page', link_title=page.link_title)
#       }) 
#   return result

# @view_config (route_name='get_page', 
#               renderer='page/view_add_edit.mako')
# @view_config (route_name='edit_page',
#               renderer='json')
# @view_config (route_name='add_page',
#               renderer='json')
# @view_config (route_name='remove_page',
#               renderer='json')
# def get_page (request):

#   route = request.matched_route.name
#   if route == "add_page":
#     untitled_number = 1
#     def link_title():
#       return "untitled_"+str(untitled_number)
#     while DBSession.query(Page).filter_by(link_title=link_title()).count() != 0L:
#       untitled_number += 1
#     page = Page(
#         link_title(),
#         "",
#         "",
#         "custom",
#         None,
#         None
#     )
#     with transaction.manager:
#       DBSession.add(page)

#     return {
#         "redirect_url": request.route_url('get_page', link_title=link_title())
#     }



#   page_link_title = request.matchdict['link_title']
#   print "page: "+page_link_title
#   resultset = DBSession.query(Page).filter_by(link_title=page_link_title)
#   if resultset.count() == 0L:
#     return HTTPNotFound("There is no page called '"+page_link_title+"'. Sorry.")
#   page = resultset.one()
#   if route == "remove_page":
#     with transaction.manager:
#       DBSession.delete(page)
#     return {
#         "redirect_url": request.route_url('home')
#         }
#   if route == "edit_page":
#     print "editing page"
#     params = request.params
#     original_link_title = page.link_title
#     print "new link_title: " + params["link_title"]
#     page.link_title   = params[u"link_title"]
#     page.title        = params[u"title"] 
#     page.subtitle     = params[u"subtitle"]
#     page.type         = params[u"type"]
#     page.content      = escapeNewlines(params[u"content"])
#     with transaction.manager:
#       DBSession.add(page) 
#     if original_link_title != params["link_title"]:
#       print "link_title changed"
#       return dict(
#           redirect_url=request.route_url('get_page', link_title=params["link_title"])
#       )
#     else:
#       return {}
#   else:
#     return {
#         "page": page,
#         "logged_in": True
#         }





from hiero.interfaces   import IHieroEntryClass
from hiero.interfaces   import IHieroCategoryClass
from hiero.interfaces   import IHieroSeriesClass
from hem.db             import get_session

class BaseController(object):
    def __init__(self, request):
        self.request = request
        self.session = get_session(request)
        self.Entry = request.registry.getUtility(IHieroEntryClass)
        self.Category = request.registry.getUtility(IHieroCategoryClass)
        self.Series = request.registry.getUtility(IHieroSeriesClass)




from hem.text                   import pluralize
from sqlalchemy.ext.declarative import declared_attr

import horus.models
import sqlalchemy as sa

ENTRY_ASSOCIATION_TABLE_NAME = 'entry_association'
ENTRY_TAG_TABLE_NAME = 'entry_tag'
ENTRY_TAG_ASSOCIATION_TABLE_NAME = 'entry_tag_association'
ENTRY_TABLE_NAME = 'entry'
SERIES_TABLE_NAME = 'series'


# These are the horus mod extensions
class BaseModel(horus.models.BaseModel):
    pass

class UserMixin(horus.models.UserMixin):
    pass

class GroupMixin(horus.models.GroupMixin):
    pass

class UserGroupMixin(horus.models.UserGroupMixin):
    pass

class ActivationMixin(horus.models.ActivationMixin):
    pass



import re
from pygments import highlight
from pygments.formatters import HtmlFormatter
from pygments.lexers import get_lexer_by_name, guess_lexer
from BeautifulSoup import BeautifulSoup

class HtmlCodeBlockFormatter(object):
    def __init__(self, text):
        self.text = text

    def pre_parse(self):
        """
        Parses out <code> blocks out of a block of text returns highlighted div
        list and a BeautifulSoup object for further parsing
        """
        index = 0
        last_index = 0
        escaped_blocks = []

        # Grab all code blocks, match <code> and <code class="foo">
        while index >= 0:
            index = self.text.find('<code', last_index)

            mid_index = self.text.find('>', index)
            last_index = self.text.find('</code>', mid_index)

            # Whats between <code> and </code>
            code_to_esc = self.text[mid_index + 1:last_index]

            lang = None

            lang_index = self.text[index:mid_index + 1].find('class=')

            if lang_index > 0:
                lang = self.text[index:mid_index + 1]
                lang = lang[lang_index + len('class="'):-2]

            if lang:
                try:
                    lexer = get_lexer_by_name(lang, stripnl=True,
                            encoding='UTF-8')
                except ValueError:
                    # We eat this exception because handling if a language
                    # wasn't found is the same as if this failed.
                    pass
            else:
                lexer = None

            if not lexer:
                lexer = get_lexer_by_name('text', stripnl=True,
                        encoding='UTF-8')

            formatter = HtmlFormatter(linenos=True,
                cssclass='source'
            )

            # Format the code as HTML and syntax highlight it
            escaped_code = highlight(code_to_esc, lexer, formatter)
            escaped_blocks.append(escaped_code)

            # re-check index< see if we have anymore
            index = self.text.find('<code', last_index)

        soup = BeautifulSoup(self.text)
        code_blocks = soup.findAll(u'code')

        # We have already formatted the code, don't allow the markup language
        # to do it a second time.
        for block in code_blocks:
            block.replaceWith(u'<code class="removed"></code>')

        # Pass it back though beautiful soup after replacing html nodes
        self.soup = BeautifulSoup(str(soup))
        self.escaped_blocks = escaped_blocks

    def post_parse(self):
        # Replace all the empty code blocks with the syntax highlighted html
        empty_code_blocks = self.soup.findAll('code', 'removed')
        for index, escaped_block in enumerate(self.escaped_blocks):
            empty_code_blocks[index].replaceWith(escaped_block)

    def get_html(self):
        self.pre_parse()
        self.post_parse()

        return str(self.soup)



import os
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        result = pytest.main(self.test_args)
        sys.exit(result)

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.md')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

requires = [
    'pyramid'
    , 'pyramid_debugtoolbar'
    , 'pyramid_tm'
    , 'waitress'
    , 'horus'
    , 'celementtree'
    , 'sqlalchemy'
    , 'psycopg2'
    , 'BeautifulSoup'
    , 'markdown'
    , 'docutils'
    , 'hem'
    , 'horus'
]

setup(
    name='hiero'
    , version='0.3.1'
    , description='hiero'
    , long_description=README + '\n\n' +  CHANGES
    , classifiers=[
        "Programming Language :: Python"
        , "Framework :: Pylons"
        , "Topic :: Internet :: WWW/HTTP"
        , "Topic :: Internet :: WWW/HTTP :: WSGI :: Application"
    ]
    , author='John Anderson'
    , author_email='sontek@gmail.com'
    , url=''
    , keywords='web pyramid pylons'
    , packages=find_packages()
    , include_package_data=True
    , zip_safe=False
    , install_requires=requires
    , cmdclass = {'test': PyTest}
    , tests_require=requires + ['pytest', 'mock', 'webtest']
    , test_suite="hiero"
    , dependency_links = [
        'https://github.com/eventray/hem/tarball/master#egg=hem-dev'
        , 'https://github.com/eventray/horus/tarball/master#egg=horus-dev'
    ]

    , entry_points = """\
    [paste.app_factory]
    main = hiero:main
    """
)




from hem.schemas        import CSRFSchema
from horus.resources    import RootFactory
from horus.interfaces   import IHorusUserClass
from hiero.interfaces   import IHieroSeriesClass
from hiero.interfaces   import IHieroCategoryClass

import colander
import deform

@colander.deferred
def owner_widget(node, kw):
    choices = [('', '')]
    request = kw.get('request')
    User = request.registry.getUtility(IHorusUserClass)

    for user in User.get_all(request):
        choices.append((str(user.pk), user.user_name))

    widget = deform.widget.SelectWidget(values=choices,
            template='hiero:templates/widgets/select_user')
    widget.request = request
    return widget

@colander.deferred
def owner_default(node, kw):
    request = kw.get('request')

    if not isinstance(request.context, RootFactory):
        return request.context.owner_pk

@colander.deferred
def series_widget(node, kw):
    choices = [('', '')]
    request = kw.get('request')
    Series = request.registry.getUtility(IHieroSeriesClass)

    for series in Series.get_all(request):
        choices.append((str(series.pk), series.title))

    widget = deform.widget.SelectWidget(values=choices,
        template='hiero:templates/widgets/select_series')
    widget.request = request

    return widget

@colander.deferred
def series_default(node, kw):
    request = kw.get('request')

    if not isinstance(request.context, RootFactory):
        return request.context.series_pk

@colander.deferred
def category_widget(node, kw):
    choices = [('', '')]
    request = kw.get('request')
    Category = request.registry.getUtility(IHieroCategoryClass)

    for category in Category.get_all(request):
        choices.append((str(category.pk), category.title))

    widget = deform.widget.SelectWidget(values=choices,
        template='hiero:templates/widgets/select_category'
    )

    widget.request = request

    return widget

@colander.deferred
def category_default(node, kw):
    request = kw.get('request')

    if not isinstance(request.context, RootFactory):
        return request.context.category_pk

@colander.deferred
def markup_widget(node, kw):
    choices = [('', '')]
    request = kw.get('request')
    formatters = ['markdown', 'htmlblock', 'rst']
    for formatter in formatters:
        choices.append((formatter, formatter))

    widget = deform.widget.SelectWidget(values=choices)
    widget.request = request

    return widget

@colander.deferred
def markup_default(node, kw):
    request = kw.get('request')

    if not isinstance(request.context, RootFactory):
        return request.context.markup


class EntryAdminSchema(CSRFSchema):

    title = colander.SchemaNode(colander.String(),
        validator=colander.Length(max=128)
    )

    slug = colander.SchemaNode(
        colander.String()
        , validator=colander.Length(max=128)
        , missing = None
    )

    markup = colander.SchemaNode(
        colander.String()
        , widget = markup_widget
        , default = markup_default
    )

    content = colander.SchemaNode(
        colander.String()
        , widget = deform.widget.TextAreaWidget(rows=25, css_class='content')
    )

    owner = colander.SchemaNode(
        colander.String()
        , widget = owner_widget
        , default = owner_default
    )

    series = colander.SchemaNode(
        colander.String()
        , widget = series_widget
        , default = series_default
        , missing = None
    )

    category = colander.SchemaNode(
        colander.String()
        , widget = category_widget
        , default = category_default
        , missing = None
    )

    is_featured = colander.SchemaNode(
        colander.Boolean()
        , missing = False
    )

    is_published = colander.SchemaNode(
            colander.Boolean()
            , missing = False
    )

    enable_comments = colander.SchemaNode(
        colander.Boolean()
        , missing = False
    )

    published_on = colander.SchemaNode(
        colander.DateTime()
        , missing = None
    )


class CategoryAdminSchema(CSRFSchema):
    title = colander.SchemaNode(colander.String(),
        validator=colander.Length(max=128)
    )

    slug = colander.SchemaNode(
        colander.String()
        , validator=colander.Length(max=128)
        , missing = None
    )

class SeriesAdminSchema(CSRFSchema):
    title = colander.SchemaNode(colander.String(),
        validator=colander.Length(max=128)
    )

    slug = colander.SchemaNode(
        colander.String()
        , validator=colander.Length(max=128)
        , missing = None
    )



#from . import Base
#
#from sqlalchemy import (
#    Column,
#    Integer,
#    UnicodeText,
#    String,
#    )
#
#class Page(Base):
#    __tablename__ = 'pages'
#    id = Column(Integer, primary_key=True)
#    link_title = Column(UnicodeText, unique=True)
#    title = Column(UnicodeText) 
#    subtitle = Column(UnicodeText) 
#    type = Column(String)
#    prev = Column(Integer)
#    next = Column(Integer)
#    content = Column(UnicodeText)
#
#    def __init__(self, link_title, title, subtitle, type, prev, next):
#        self.link_title = link_title
#        self.title = title
#        self.subtitle = subtitle
#        self.type = type
#        self.prev = prev
#        self.next = next
#        self.content = u""



from hiero.views            import BaseController
from hiero.schemas.blog     import EntryAdminSchema
from hiero.schemas.blog     import CategoryAdminSchema
from hiero.schemas.blog     import SeriesAdminSchema
from hiero.forms            import HieroForm
from hiero.formatters       import get_formatter
from horus.resources        import RootFactory
from pyramid.view           import view_config
from pyramid.view           import view_defaults
from pyramid.httpexceptions import HTTPFound
from sqlalchemy.orm.exc     import NoResultFound
from pyramid.i18n           import TranslationStringFactory
import deform
import logging

logger = logging.getLogger(__name__)

_ = TranslationStringFactory('hiero')

class EntryController(BaseController):
    @view_config(
        route_name='hiero_entry_index'
        , renderer='hiero:templates/blog_index.mako'
    )
    @view_config(
        route_name='hiero_entry_index_paged'
        , renderer='hiero:templates/blog_index.mako'
    )
    def index(self):
        """ View that lists and pages all the entries """
        page = int(self.request.matchdict.get('page', 1))

        query = self.Entry.get_all_active(self.request, page=page)

        return {
            'entries': query.all()
        }

    @view_config(
        route_name='hiero_entry_category'
        , renderer='hiero:templates/blog_index.mako'
    )
    def category_index(self):
        """ View that lists and pages all the entries 
        for a specific category 
        """
        page = int(self.request.matchdict.get('page', 1))
        slug = self.request.matchdict.get('slug')

        query = self.Entry.get_all_active(self.request, page=page)
        query = query.join(self.Category)
        query = query.filter(self.Category.slug == slug)

        return {
            'entries': query.all()
        }


    @view_config(
        route_name='hiero_entry_series'
        , renderer='hiero:templates/blog_index.mako'
    )
    def series_index(self):
        """ View that lists and pages all the entries 
        in a series 
        """
        page = int(self.request.matchdict.get('page', 1))
        slug = self.request.matchdict.get('slug')

        query = self.Entry.get_all_active(self.request, page=page)
        query = query.join(self.Series)
        query = query.filter(self.Slug.slug == slug)

        return {
            'entries': query.all()
        }

    @view_config(
        route_name='hiero_entry_detail'
        , renderer='hiero:templates/entry_detail.mako'
    )
    def detail(self):
        """ View that is a detailed view of a single entry """
        slug = self.request.matchdict.get('slug', None)

        if slug:
            query = self.Entry.get_by_slug(self.request, slug)

            try:
                result = query.one()
                return dict(entry=result)
            except NoResultFound as exc:
                logger.debug('Could not find slug %s' % slug)
                raise


    @view_config(
        route_name='hiero_entry_search'
        , renderer='hiero:templates/detail.mako'
    )
    def search(self):
        """ View that allows you to search entries """
        pass

@view_defaults(permission='group:admin')
class AdminEntryController(BaseController):
    @view_config(
        route_name='hiero_admin_index'
        , renderer='hiero:templates/admin/index.mako'
    )
    def index(self):
        return {}

    @view_config(
        route_name='hiero_admin_entry_index'
        , renderer='hiero:templates/admin/entry_index.mako'
    )
    @view_config(
        route_name='hiero_admin_entry_index_paged'
        , renderer='hiero:templates/admin/entry_index.mako'
    )
    def entry_index(self):
        """ View that lists and pages all the entries for admins """
        page = int(self.request.matchdict.get('page', 1))

        query = self.Entry.get_all(self.request, page=page)

        return dict(entries=query.all())

    @view_config(
            route_name='hiero_admin_entry_create'
            , renderer='hiero:templates/admin/edit_entry.mako'
    )
    @view_config(
            route_name='hiero_admin_entry_edit'
            , renderer='hiero:templates/admin/edit_entry.mako'
    )
    def create_entry(self):
        schema = EntryAdminSchema()
        schema = schema.bind(request=self.request)
        form = HieroForm(self.request, schema)

        if self.request.method == 'GET':
            if isinstance(self.request.context, RootFactory):
                return dict(form=form)
            else:
                return dict(
                    form=form,
                    appstruct = self.request.context.__json__()
                )
        else:
            try:
                controls = self.request.POST.items()
                captured = form.validate(controls)
            except deform.ValidationFailure, e:
                return dict(form=e, errors=e.error.children)


            if isinstance(self.request.context, RootFactory):
                entry = self.Entry()
            else:
                entry = self.request.context

            entry.title = captured['title']
            entry.owner_pk = captured['owner']
            entry.content = captured['content']
            entry.markup = captured['markup']

            formatter = get_formatter(captured['markup'])

            if formatter:
                entry.html_content = formatter(entry.content).get_html()
            else:
                entry.html_content = entry.content

            entry.category_pk = captured['category']
            entry.series_pk = captured['series']
            entry.is_featured = captured['is_featured']
            entry.is_published = captured['is_published']
            entry.enable_comments = captured['enable_comments']
            published_on = captured['published_on']
            entry.published_on = published_on.replace(tzinfo=None)


            if captured['slug']:
                entry.slug = captured['slug']

            self.session.add(entry)

            self.request.session.flash(_(u'The entry was created'), 'success')

            return HTTPFound(
                location=self.request.route_url('hiero_admin_entry_index')
            )

    @view_config(
            route_name='hiero_admin_category_index'
            , renderer='hiero:templates/admin/category_index.mako'
    )
    @view_config(
            route_name='hiero_admin_category_index_paged'
            , renderer='hiero:templates/admin/category_index.mako'
    )
    def category_index(self):
        page = int(self.request.matchdict.get('page', 1))

        query = self.Category.get_all(self.request, page=page)

        return dict(categories=query.all())

    @view_config(
            route_name='hiero_admin_category_create'
            , renderer='hiero:templates/admin/edit_category.mako'
    )
    @view_config(
            route_name='hiero_admin_category_edit'
            , renderer='hiero:templates/admin/edit_category.mako'
    )
    def create_category(self):
        schema = CategoryAdminSchema()
        schema = schema.bind(request=self.request)
        form = HieroForm(self.request, schema)

        if self.request.method == 'GET':
            if isinstance(self.request.context, RootFactory):
                return dict(form=form)
            else:
                return dict(
                    form=form,
                    appstruct = self.request.context.__json__()
                )
        else:
            try:
                controls = self.request.POST.items()
                captured = form.validate(controls)
            except deform.ValidationFailure, e:
                return dict(form=e, errors=e.error.children)


            if isinstance(self.request.context, RootFactory):
                category = self.Category()
            else:
                category = self.request.context

            category.title = captured['title']

            if captured['slug']:
                category.slug = captured['slug']

            self.session.add(category)

            self.request.session.flash(_(u'The category was created'), 'success')

            return HTTPFound(
                location=self.request.route_url('hiero_admin_category_index')
            )


    @view_config(
            route_name='hiero_admin_series_index'
            , renderer='hiero:templates/admin/series_index.mako'
    )
    @view_config(
            route_name='hiero_admin_series_index_paged'
            , renderer='hiero:templates/admin/series_index.mako'
    )
    def series_index(self):
        page = int(self.request.matchdict.get('page', 1))

        query = self.Series.get_all(self.request, page=page)

        return dict(series=query.all())

    @view_config(
            route_name='hiero_admin_series_create'
            , renderer='hiero:templates/admin/edit_series.mako'
    )
    @view_config(
            route_name='hiero_admin_series_edit'
            , renderer='hiero:templates/admin/edit_series.mako'
    )
    def create_series(self):
        schema = SeriesAdminSchema()
        schema = schema.bind(request=self.request)
        form = HieroForm(self.request, schema)

        if self.request.method == 'GET':
            if isinstance(self.request.context, RootFactory):
                return dict(form=form)
            else:
                return dict(
                    form=form,
                    appstruct = self.request.context.__json__()
                )
        else:
            try:
                controls = self.request.POST.items()
                captured = form.validate(controls)
            except deform.ValidationFailure, e:
                return dict(form=e, errors=e.error.children)


            if isinstance(self.request.context, RootFactory):
                series = self.Series()
            else:
                series = self.request.context

            series.title = captured['title']

            if captured['slug']:
                series.slug = captured['slug']

            self.session.add(series)

            self.request.session.flash(_(u'The series was created'), 'success')

            return HTTPFound(
                location=self.request.route_url('hiero_admin_series_index')
            )
